@@ -17,13 +17,13 @@ from typing import Any, Callable, DefaultDict, Dict, Generic, Iterable, List, Op
 import numpy as np
 import pandas as pd
 from sklearn.base import BaseEstimator
-from typing_extensions import Annotated, Literal, get_args, get_origin
+from typing_extensions import Annotated, Literal, Self, get_args, get_origin
 
 from tpcp._parameters import _ParaTypes
 from tpcp.exceptions import MutableDefaultsError, PotentialUserErrorWarning, ValidationError
 
 T = TypeVar("T")
-BaseTpcpObjectObj_ = TypeVar("BaseTpcpObjectObj_", bound="BaseTpcpObject")
+BaseTpcpObjectObjT = TypeVar("BaseTpcpObjectObjT", bound="BaseTpcpObject")
 
 
 class _Nothing:
@@ -227,14 +227,14 @@ class BaseTpcpObject(_BaseTpcpObject, _skip_validation=True):
         """
         return _get_params(self, deep)
 
-    def set_params(self: BaseTpcpObjectObj_, **params: Any) -> BaseTpcpObjectObj_:
+    def set_params(self: Self, **params: Any) -> Self:
         """Set the parameters of this Algorithm.
 
         To set parameters of nested objects use `nested_object_name__para_name=`.
         """
         return _set_params(self, **params)
 
-    def clone(self: BaseTpcpObjectObj_) -> BaseTpcpObjectObj_:
+    def clone(self) -> Self:
         """Create a new instance of the class with all parameters copied over.
 
         This will create a new instance of the class itself and all nested objects
@@ -271,7 +271,7 @@ def _get_params(instance: _BaseTpcpObject, deep: bool = True) -> Dict[str, Any]:
     return out
 
 
-def _set_params(instance: BaseTpcpObjectObj_, **params: Any) -> BaseTpcpObjectObj_:
+def _set_params(instance: BaseTpcpObjectObjT, **params: Any) -> BaseTpcpObjectObjT:
     """Set the parameters of an instance.
 
     To set parameters of nested objects use `nested_object_name__para_name=`.
@@ -492,9 +492,14 @@ class CloneFactory(BaseFactory, Generic[T]):
         return clone(self.default_value)
 
 
-def cf(default_value: T) -> CloneFactory[T]:  # noqa: invalid-name
+# The typing here is obviously wrong and a hack to make object that are wrapped by clone factory seem to be just of
+# the same type.
+# Ideally clone factory would be some generic type of proxy, but it isn't.
+# Therefore, we fake the return type, so that external type checking in user code works without any errors.
+# As long as `cf` is used for its intended purpose that should not matter much.
+def cf(default_value: T) -> T:  # noqa: invalid-name
     """Wrap mutable default value with the `CloneFactory`.
 
     This is basically an alias for :class:`~tpcp.CloneFactory`
     """
-    return CloneFactory(default_value)
+    return CloneFactory(default_value)  # type: ignore
